/**
 * @file   rusage.c
 * @author David Bremner <bremner@unb.ca>
 * @date   Fri Dec  6 14:45:31 2002
 * 
 * @brief  interface to system resource monitoring
 * 
 * 
 */


#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#include "error_message.h"
#include "rusage.h"
#define timeval_to_double(tv) ((double)tv.tv_sec+(double)tv.tv_usec/1000000)
void ru_print(){
  struct rusage ru;

  if (getrusage(RUSAGE_SELF,&ru)!=0)
    err_fatal(HERE,"getrusage");

  fprintf(stderr,"User Time: %9.3f\n", timeval_to_double(ru.ru_utime));
  fprintf(stderr,"System Time: %9.3f\n", timeval_to_double(ru.ru_stime) );
  
}

double ru_utime(){
  struct rusage ru;

  if (getrusage(RUSAGE_SELF,&ru)!=0)
    err_fatal(HERE,"getrusage");

  return timeval_to_double(ru.ru_utime);
}


double ru_stime(){
  struct rusage ru;

  if (getrusage(RUSAGE_SELF,&ru)!=0)
    err_fatal(HERE,"getrusage");

  return timeval_to_double(ru.ru_stime);
}

double ru_cputime(){
  struct rusage ru;

  if (getrusage(RUSAGE_SELF,&ru)!=0)
    err_fatal(HERE,"getrusage");

  return timeval_to_double(ru.ru_utime)+timeval_to_double(ru.ru_stime);
}
