/* routines to interface to zram checkpointing */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef USE_ZRAM
#include <stdio.h>
#include <stdlib.h>
#include <z_io.h>
#include <string.h>
#include <unistd.h>
#include "nalloc.h"
#include "oms_error.h"
#include "checkpoint.h"

char *chk_global_name(char *base){
  char *name;
  
  name=malloc(BUFSIZ);
  
  snprintf(name,BUFSIZ,"%s-globals",base);

  return name;
}

void chk_save_global(char *base, na_node_p global){

  FILE *gf;
  char *name;

  name=chk_global_name(base);
  gf=fopen(name,"w");
  if (gf==NULL)
    oms_fatal(HERE,"fopen/w: %s",name);
  z_WriteNode(gf,(z_NodeP) global);
  fclose(gf);
  free(name);
}

na_node_p chk_restore_global(char *base){
  
  FILE *gf;
  char *name;
  na_node_p new_global;

  name=chk_global_name(base);
  gf=fopen(name,"r");
  if (gf==NULL)
    oms_fatal(HERE,"fopen/r: %s",name);
  new_global=(na_node_p)z_ReadNode(gf);
  fclose(gf);
  free(name);
  na_fixup(new_global);
  return new_global;
}

void chk_set_default_name(char *loc, size_t len){
  
  getcwd(loc,PATH_MAX);
  strncat(loc,"/_checkXXXXXX",PATH_MAX);
  mkstemp(loc);
}
#else
#include <stdlib.h>
#include "nalloc.h"
char *chk_global_name(char *base){
}

void chk_save_global(char *base, na_node_p global){
}

na_node_p chk_restore_global(char *base){
}

void chk_set_default_name(char *loc, size_t len){
  
}
#endif
