/**
 * @file   nu_pluecker.c
 * @author David Bremner <bremner@unb.ca>
 * @date   Sun Dec 22 15:03:23 2002
 * 
 * @brief  non-uniform pluecker equation checking
 * 
 * 
 */

#include "debug.h"
#include "oms_types.h"
#include "basis_equation.h"
#include "nu_sign.h"
#include "nu_sign_set.h"
#include "pluecker_action.h"
#include "sa32_array.h"

extern pl_token_t pluecker_check[];
extern pl_action_t pl_actions[];


bool npl_traverse(array_index_t k,
		   be_block_t *BE,nu_sign_set_p forced_signs,
		   sarray_p chi,sarray_p pluecker_state) {
  int elem;


  elem=BE->last[k];		/* @elem@ points to the last occur. of @k@ */
  err_assert( elem >= -1 && elem < BE->rows*BE->cols );
 


  while ( elem>=0  ){
    array_index_t row,col,row_start;
    nu_sign_t new_sign;
    uint32_t row_state;
    pl_token_t action_token;
    be_coeff_p Eqn,entry;

    entry=BE->eqn+elem;

    row=be_row(BE,elem);
    row_start=be_row_start(BE,elem);
    col=be_col(BE,elem);

    Eqn=BE->eqn+row_start;     

    new_sign=sa_read(chi,be_bindex_raw(BE,elem));

    /*
      note, the negation of the middle pair is
      taken care of in the table; i.e. the row 
      state should correspond precisely to the chirotope.
    */
    

    
    row_state=sa_set_bits(pluecker_state,row, new_sign,
			  col*SIGN_BITWIDTH,SIGN_BITWIDTH);

    action_token=pluecker_check[row_state];

    err_debug(DB_PLUECKER,(HERE,"row_state=%o action=%d\n",row_state,action_token));

    if (action_token==0)
      ERR_RETURN( FALSE);

    if (action_token>1){
      int i;

      for (i=0; i<pl_actions[action_token].count; i++){
	int off=pl_actions[action_token].setting[i].array_index_t;
	int bindex=be_bindex_raw(BE,row_start+off);
	nu_sign_t new_sign=pl_actions[action_token].setting[i].sign;
	nu_sign_t cur_sign=sa_read(chi,bindex);
	
	if (!sign_compatible(new_sign,cur_sign))
	  ERR_RETURN(FALSE);

	if (!sign_subset(cur_sign,new_sign)){
	  err_debug(DB_PLUECKER|DB_FORCE,(HERE,"row[%d] state %o action %d forcing chi[%d] to %s",row,row_state,action_token,bindex,sign_name[new_sign]));
	  if (!nss_add(forced_signs,bindex,new_sign))
	    ERR_RETURN(FALSE);
	}
      }
    }
    elem=BE->eqn[elem].prev;	/* 1.8 % */
  }
  ERR_RETURN(TRUE);
}

