/* cis_base.h

   Connected induced subgraphs
   Ambros Marzetta

 * $Source$
 * $Revision: 2732 $
 * $Date: 2000-11-15 17:14:52 +0100 (Wed, 15 Nov 2000) $
 * $Log$
 * Revision 1.2  2000/11/15 16:14:51  bremner
 * committing patched version of zram source
 *
 * Revision 1.1  1996/06/03  18:02:14  marzetta
 * Initial revision
 *
 */

#include "z_base.h"
#include "z_adjlist.h"
#include "z_ps2d.h"
#include "z_rs.h"

#define cis_pointT	uint16	/* number of points */
#define cis_edgeT	adjpointT /* number of edges */
#define MAXDEGREE	1000	/* degree of graph, used for static arrays */

extern int cis_NodeTag, cis_GlobalTag;

typedef struct cis_Node {
  Z_NODE_HEADER;		/* len; tag = cis_NodeTag */
  cis_pointT numAct;		/* number of points in induced subgraph */
  boolean pt[1];		/* variable size: numPt */
} cis_Node, *cis_NodeP;

#define is_root_node(x) (x->numAct == 0)

typedef struct cis_Global {
  Z_NODE_HEADER;		/* len; tag = cis_GlobalTag */
  z_adjlistP cis_adjlist;	/* adjacency lists */
  z_ps2dP cis_coord;		/* coordinates */
  cis_pointT numPt;		/* number of points */
} cis_Global, *cis_GlobalP;

#define cis_adjPt(gp,pt)	(adjlist_adjPt(gp->cis_adjlist, pt))
				/* adjacency list of point pt */
#define cis_coord_x(gp, i)	ps2d_coord_x(gp->cis_coord, i)
#define cis_coord_y(gp, i)	ps2d_coord_y(gp->cis_coord, i)

cis_GlobalP cis_Read(FILE *infile);
/* Read Graph from infile and allocate a global node */
/* Should test whether graph is connected. */
/* Should test for parallel edges */

cis_NodeP cis_EmptySubgraph(cis_GlobalP global);

void cis_PSHeader(cis_GlobalP global, FILE *psfile, float xsize, float ysize);

void cis_PSPicture(cis_GlobalP global, cis_NodeP node, FILE *psfile,
		   float xsize, float ysize);

boolean cis_Equal(cis_GlobalP global, cis_NodeP x, cis_NodeP y);

void cis_Print(cis_GlobalP global, cis_NodeP node);

cis_NodeP cis_f(cis_GlobalP global, cis_NodeP node);

cis_NodeP cis_adj_oracle(cis_GlobalP global, cis_NodeP node, uint32 k);

boolean cis_is_root_node(cis_GlobalP global, cis_NodeP this);

unsigned int cis_Delta(cis_GlobalP global, cis_NodeP this);

void cis_base_Init(void);
