/* cis_main.c

   Connected induced subgraphs: main program
   Ambros Marzetta

   95-06-12
 * $Source$
 * $Revision: 2732 $
 * $Date: 2000-11-15 17:14:52 +0100 (Wed, 15 Nov 2000) $
 * $Log$
 * Revision 1.2  2000/11/15 16:14:51  bremner
 * committing patched version of zram source
 *
 * Revision 1.2  1996/09/17  13:34:14  marzetta
 * changed type of getopt result to int
 *
 * Revision 1.1  1996/06/03  18:01:57  marzetta
 * Initial revision
 *
 */

#include <stdlib.h>		/* exit() */
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include "z_enumps.h"
#include "z_par.h"		/* z_myproc */
#include "z_time.h"
#include "cis_base.h"
#include "z_io.h"		/* getopt() */

static z_rs_ProcSet cis_procset1, cis_procset2, cis_procset3;
static int cis_procsetnum1, cis_procsetnum2, cis_procsetnum3;

void initproc(void)
{
  /* This function is called once on every processing element. */
  cis_base_Init();
  z_rs_Init();
  z_enumps_init();
  
  cis_procset1.rs_Equal = (z_EqualProc)cis_Equal;
  cis_procset1.rs_Condition = Z_CONDITION_ALWAYS;
  cis_procset1.rs_Result = (z_NodeProc)cis_Print;
  cis_procset1.rs_F = (z_rs_FProc)cis_f;
  cis_procset1.rs_AdjOracle = (z_rs_AdjOracleProc)cis_adj_oracle;
  cis_procset1.rs_IsRoot = (z_ConditionProc)cis_is_root_node;
  cis_procset1.rs_Delta = (z_rs_DeltaProc)cis_Delta;
  cis_procset1.rs_FindChildNumber = NULL;
  cis_procset1.rs_Forward = NULL;
  cis_procset1.rs_NextChild = NULL;
  cis_procsetnum1 = z_InstallProcSet((z_ProcSetP) &cis_procset1);

  cis_procset2 = cis_procset1;
  cis_procset2.rs_Result = z_Save;
  cis_procsetnum2 = z_InstallProcSet((z_ProcSetP) &cis_procset2);
  
  cis_procset3 = cis_procset1;
  cis_procset3.rs_Result = NULL;
  cis_procsetnum3 = z_InstallProcSet((z_ProcSetP) &cis_procset3);
} /* initproc */

static int Usage(const char *name)
{
  fprintf(stderr, "Usage: %s [-1npt]\n\n", name);
  fprintf(stderr, "Options: -1  sequential\n");
  fprintf(stderr, "         -h  help (produce this message)\n");
  fprintf(stderr, "         -n  no output (implies time measurement)\n");
  fprintf(stderr, "         -p  PostScript output\n");
  fprintf(stderr, "         -t  time measurement\n");
  return 4;
} /* Usage */

int mmain(int argc, char **argv, char **envp)
{
  cis_GlobalP global;
  cis_NodeP root;
  double time;
  enum {ot_normal, postScript, noOutput} outputType = ot_normal;
  boolean showTime=0;
  boolean sequential=0;
  int procsetnum;
  int c;

  while ((c=getopt(argc, argv, "1hnpt")) != -1)
    switch(c) {
    case 'p':
      outputType = postScript;
      sequential = 1;
      break;
    case 't':
      showTime = 1;
      break;
    case 'n':
      outputType = noOutput;
      showTime = 1;
      break;
    case '1':
      sequential = 1;
      break;
    default:
      return Usage(argv[0]);
    }
  
  if (argc != optind) return Usage(argv[0]);
  
  global = cis_Read(stdin);
  if (!global) return 1;
  root = cis_EmptySubgraph(global);

  switch (outputType) {
  case noOutput:
    procsetnum = cis_procsetnum3; /* no output */
    break;
  case postScript:
    procsetnum = cis_procsetnum2; /* save */
    z_enumps_Open("/tmp/pm");
    break;
  default:
    procsetnum = cis_procsetnum1; /* print */
    break;
  }
  
  time = z_Gtime();
  z_rs_Enumerate(procsetnum, (z_lb_GlobalP) global,
		 (z_NodeP)root, sequential);
  time = z_Gtime() - time;

  if (outputType == postScript) {
    z_PrintPS((z_NodeP)global, stdout,
	      (z_enumps_HeaderProc)cis_PSHeader, (z_enumps_PictureProc)cis_PSPicture);
  }
  if (showTime)
    if (sequential)
      printf("Time:%7.2f (sequential)\n",
	     time);
    else
      printf("Time:%7.2f (%2d processor%s\n",
	     time, z_numprocs, z_numprocs>1?"s)":") ");
  return 0;
} /* mmain */
